from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.data_element_permissible_values_text_range_text_range import (
    DataElementPermissibleValuesTextRangeTextRange,
)


T = TypeVar("T", bound="DataElementPermissibleValuesTextRange")


@_attrs_define
class DataElementPermissibleValuesTextRange:
    """DataElementPermissibleValuesTextRange model

    Attributes:
        text_range (DataElementPermissibleValuesTextRangeTextRange): Controls on the textual domain of the data.
    """

    text_range: "DataElementPermissibleValuesTextRangeTextRange"

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        text_range = self.text_range.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "textRange": text_range,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`DataElementPermissibleValuesTextRange` from a dict"""
        d = src_dict.copy()
        text_range = DataElementPermissibleValuesTextRangeTextRange.from_dict(
            d.pop("textRange")
        )

        data_element_permissible_values_text_range = cls(
            text_range=text_range,
        )

        return data_element_permissible_values_text_range



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.pagination import Pagination
from ..models.project import Project


T = TypeVar("T", bound="ProjectsGetProjectsProjectList")


@_attrs_define
class ProjectsGetProjectsProjectList:
    """ProjectsGetProjectsProjectList model

    Attributes:
        pagination (Pagination):
        projects (List['Project']):
    """

    pagination: "Pagination"
    projects: List["Project"]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        pagination = self.pagination.to_dict()
        projects = []
        for projects_item_data in self.projects:
            projects_item = projects_item_data.to_dict()
            projects.append(projects_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "pagination": pagination,
                "projects": projects,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`ProjectsGetProjectsProjectList` from a dict"""
        d = src_dict.copy()
        pagination = Pagination.from_dict(d.pop("pagination"))

        projects = []
        _projects = d.pop("projects")
        for projects_item_data in _projects:
            projects_item = Project.from_dict(projects_item_data)

            projects.append(projects_item)

        projects_get_projects_project_list = cls(
            pagination=pagination,
            projects=projects,
        )

        return projects_get_projects_project_list



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.system_configuration_authentication import (
    SystemConfigurationAuthentication,
)
from ..models.system_configuration_cache import SystemConfigurationCache
from ..models.system_configuration_datastores import SystemConfigurationDatastores
from ..models.system_configuration_workers import SystemConfigurationWorkers


T = TypeVar("T", bound="SystemConfiguration")


@_attrs_define
class SystemConfiguration:
    """SystemConfiguration model

    Attributes:
        authentication (SystemConfigurationAuthentication):
        cache (SystemConfigurationCache):
        datastores (SystemConfigurationDatastores):
        workers (SystemConfigurationWorkers):
    """

    authentication: "SystemConfigurationAuthentication"
    cache: "SystemConfigurationCache"
    datastores: "SystemConfigurationDatastores"
    workers: "SystemConfigurationWorkers"

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        authentication = self.authentication.to_dict()
        cache = self.cache.to_dict()
        datastores = self.datastores.to_dict()
        workers = self.workers.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "authentication": authentication,
                "cache": cache,
                "datastores": datastores,
                "workers": workers,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SystemConfiguration` from a dict"""
        d = src_dict.copy()
        authentication = SystemConfigurationAuthentication.from_dict(
            d.pop("authentication")
        )

        cache = SystemConfigurationCache.from_dict(d.pop("cache"))

        datastores = SystemConfigurationDatastores.from_dict(d.pop("datastores"))

        workers = SystemConfigurationWorkers.from_dict(d.pop("workers"))

        system_configuration = cls(
            authentication=authentication,
            cache=cache,
            datastores=datastores,
            workers=workers,
        )

        return system_configuration



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="DataElementPermissibleValuesNumberRangeNumberRange")


@_attrs_define
class DataElementPermissibleValuesNumberRangeNumberRange:
    """Controls on the numerical domain of the data.

    Attributes:
        is_integer (Union[Unset, bool]): True if the number should be an integer.
        maximum (Union[Unset, float]):
        meaning (Union[Unset, str]): A description of this range's meaning.
        minimum (Union[Unset, float]):
    """

    is_integer: Union[Unset, bool] = UNSET
    maximum: Union[Unset, float] = UNSET
    meaning: Union[Unset, str] = UNSET
    minimum: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        is_integer = self.is_integer
        maximum = self.maximum
        meaning = self.meaning
        minimum = self.minimum

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if is_integer is not UNSET:
            field_dict["isInteger"] = is_integer
        if maximum is not UNSET:
            field_dict["maximum"] = maximum
        if meaning is not UNSET:
            field_dict["meaning"] = meaning
        if minimum is not UNSET:
            field_dict["minimum"] = minimum

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`DataElementPermissibleValuesNumberRangeNumberRange` from a dict"""
        d = src_dict.copy()
        is_integer = d.pop("isInteger", UNSET)

        maximum = d.pop("maximum", UNSET)

        meaning = d.pop("meaning", UNSET)

        minimum = d.pop("minimum", UNSET)

        data_element_permissible_values_number_range_number_range = cls(
            is_integer=is_integer,
            maximum=maximum,
            meaning=meaning,
            minimum=minimum,
        )

        return data_element_permissible_values_number_range_number_range



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar("T", bound="AuthSignoutNoData")


@_attrs_define
class AuthSignoutNoData:
    """AuthSignoutNoData model"""

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""

        field_dict: Dict[str, Any] = {}
        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`AuthSignoutNoData` from a dict"""
        src_dict.copy()
        auth_signout_no_data = cls()

        return auth_signout_no_data



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.collection import Collection
from ..models.pagination import Pagination


T = TypeVar("T", bound="CollectionsGetCollectionsCollectionList")


@_attrs_define
class CollectionsGetCollectionsCollectionList:
    """CollectionsGetCollectionsCollectionList model

    Attributes:
        collections (List['Collection']):
        pagination (Pagination):
    """

    collections: List["Collection"]
    pagination: "Pagination"

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        collections = []
        for collections_item_data in self.collections:
            collections_item = collections_item_data.to_dict()
            collections.append(collections_item)

        pagination = self.pagination.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "collections": collections,
                "pagination": pagination,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionsGetCollectionsCollectionList` from a dict"""
        d = src_dict.copy()
        collections = []
        _collections = d.pop("collections")
        for collections_item_data in _collections:
            collections_item = Collection.from_dict(collections_item_data)

            collections.append(collections_item)

        pagination = Pagination.from_dict(d.pop("pagination"))

        collections_get_collections_collection_list = cls(
            collections=collections,
            pagination=pagination,
        )

        return collections_get_collections_collection_list



from pypigeon.pigeon_core.api.account import account_create_account
from pypigeon.pigeon_core.models import AccountCreateAccountBody

from .base_commands import BaseCommands


class AccountsCommands(BaseCommands):
    """Operations on accounts"""

    @BaseCommands._with_arg("name")
    def new(self) -> None:
        """Create a new account."""
        rv = account_create_account.sync(
            body=AccountCreateAccountBody(name=self.args.name), client=self.core
        )

        self._output(rv.to_dict())



import datetime
from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="Account")


@_attrs_define
class Account:
    """Account model

    Attributes:
        created_on (datetime.datetime):
        id (str):
        is_personal (bool):
        name (str):
        members_group_id (Union[Unset, str]):
    """

    created_on: datetime.datetime
    id: str
    is_personal: bool
    name: str
    members_group_id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        created_on = self.created_on.isoformat()
        id = self.id
        is_personal = self.is_personal
        name = self.name
        members_group_id = self.members_group_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "createdOn": created_on,
                "id": id,
                "isPersonal": is_personal,
                "name": name,
            }
        )
        if members_group_id is not UNSET:
            field_dict["membersGroupId"] = members_group_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`Account` from a dict"""
        d = src_dict.copy()
        created_on = isoparse(d.pop("createdOn"))

        id = d.pop("id")

        is_personal = d.pop("isPersonal")

        name = d.pop("name")

        members_group_id = d.pop("membersGroupId", UNSET)

        account = cls(
            created_on=created_on,
            id=id,
            is_personal=is_personal,
            name=name,
            members_group_id=members_group_id,
        )

        return account



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.authorized_actions_item import AuthorizedActionsItem
from ..models.collection_authorization_set_identity_action import (
    CollectionAuthorizationSetIdentityAction,
)
from ..models.collection_role import CollectionRole
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="CollectionAuthorizationSetIdentity")


@_attrs_define
class CollectionAuthorizationSetIdentity:
    """CollectionAuthorizationSetIdentity model

    Attributes:
        action (CollectionAuthorizationSetIdentityAction):
        identity (str):
        actions (Union[Unset, List[AuthorizedActionsItem]]):
        role (Union[Unset, CollectionRole]):
    """

    action: CollectionAuthorizationSetIdentityAction
    identity: str
    actions: Union[Unset, List[AuthorizedActionsItem]] = UNSET
    role: Union[Unset, CollectionRole] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        action = self.action.value
        identity = self.identity
        actions: Union[Unset, List[str]] = UNSET
        if not isinstance(self.actions, Unset):
            actions = []
            for componentsschemasauthorized_actions_item_data in self.actions:
                componentsschemasauthorized_actions_item = (
                    componentsschemasauthorized_actions_item_data.value
                )
                actions.append(componentsschemasauthorized_actions_item)

        role: Union[Unset, str] = UNSET
        if not isinstance(self.role, Unset):
            role = self.role.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "action": action,
                "identity": identity,
            }
        )
        if actions is not UNSET:
            field_dict["actions"] = actions
        if role is not UNSET:
            field_dict["role"] = role

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionAuthorizationSetIdentity` from a dict"""
        d = src_dict.copy()
        action = CollectionAuthorizationSetIdentityAction(d.pop("action"))

        identity = d.pop("identity")

        actions = []
        _actions = d.pop("actions", UNSET)
        for componentsschemasauthorized_actions_item_data in _actions or []:
            componentsschemasauthorized_actions_item = AuthorizedActionsItem(
                componentsschemasauthorized_actions_item_data
            )

            actions.append(componentsschemasauthorized_actions_item)

        _role = d.pop("role", UNSET)
        role: Union[Unset, CollectionRole]
        if isinstance(_role, Unset):
            role = UNSET
        else:
            role = CollectionRole(_role)

        collection_authorization_set_identity = cls(
            action=action,
            identity=identity,
            actions=actions,
            role=role,
        )

        return collection_authorization_set_identity



from datetime import datetime

import pandas as pd
import pytest
from pypigeon.collections import PigeonCollection
from pypigeon.pigeon_core.models import Collection


@pytest.fixture
def collection(client):
    m = Collection(
        access_level="private",
        account_id="zzz",
        created_on=datetime.now(),
        description="",
        id="xyz",
        name="XYZ",
        version_id="LIVE",
        metadata={},
    )
    return PigeonCollection(m, client)


def test_list_folder(httpx_mock, collection):
    # list top level
    httpx_mock.add_response(
        url="https://pigeon.test/api/v1/collections/xyz/LIVE/items?in_folder=ROOT",
        json={
            "items": [
                {
                    "id": "f",
                    "name": "foo",
                    "type": "file",
                    "createdOn": datetime.now().isoformat(),
                },
                {
                    "id": "b",
                    "name": "bar",
                    "type": "link",
                    "createdOn": datetime.now().isoformat(),
                },
                {
                    "id": "z",
                    "name": "baz",
                    "type": "folder",
                    "createdOn": datetime.now().isoformat(),
                },
            ],
            "pagination": {
                "total_items": 3,
                "items_per_page": 50,
                "next_page_url": None,
            },
        },
    )
    assert [i.name for i in collection] == ["foo", "bar", "baz"]


def test_get_and_list_folder(httpx_mock, collection):
    # go down a folder and list it
    httpx_mock.add_response(
        url=(
            "https://pigeon.test/api/v1/collections/xyz/LIVE"
            "/items?in_folder=ROOT&name=baz&type=folder"
        ),
        json={
            "items": [
                {
                    "id": "z",
                    "name": "baz",
                    "type": "folder",
                    "createdOn": datetime.now().isoformat(),
                },
            ],
            "pagination": {
                "total_items": 1,
                "items_per_page": 50,
                "next_page_url": None,
            },
        },
    )
    httpx_mock.add_response(
        url="https://pigeon.test/api/v1/collections/xyz/LIVE/items?in_path=baz",
        json={
            "items": [
                {
                    "id": "1",
                    "name": "one",
                    "type": "file",
                    "createdOn": datetime.now().isoformat(),
                },
                {
                    "id": "2",
                    "name": "two",
                    "type": "file",
                    "createdOn": datetime.now().isoformat(),
                },
            ],
            "pagination": {
                "total_items": 2,
                "items_per_page": 50,
                "next_page_url": None,
            },
        },
    )

    f = collection.folder("baz")
    assert f.name == "baz"
    assert [i.name for i in f] == ["one", "two"]
    assert [i.in_folder for i in f] == [f, f]


def test_walk_folder(httpx_mock, collection):
    # walk all items
    httpx_mock.add_response(
        url="https://pigeon.test/api/v1/collections/xyz/LIVE/items?in_folder=ROOT",
        json={
            "items": [
                {
                    "id": "f",
                    "name": "foo",
                    "type": "file",
                    "createdOn": datetime.now().isoformat(),
                },
                {
                    "id": "b",
                    "name": "bar",
                    "type": "link",
                    "createdOn": datetime.now().isoformat(),
                },
                {
                    "id": "z",
                    "name": "baz",
                    "type": "folder",
                    "createdOn": datetime.now().isoformat(),
                },
            ],
            "pagination": {
                "total_items": 3,
                "items_per_page": 50,
                "next_page_url": None,
            },
        },
    )
    httpx_mock.add_response(
        url="https://pigeon.test/api/v1/collections/xyz/LIVE/items?in_path=baz",
        json={
            "items": [
                {
                    "id": "1",
                    "name": "one",
                    "type": "file",
                    "createdOn": datetime.now().isoformat(),
                },
                {
                    "id": "2",
                    "name": "two",
                    "type": "file",
                    "createdOn": datetime.now().isoformat(),
                },
            ],
            "pagination": {
                "total_items": 2,
                "items_per_page": 50,
                "next_page_url": None,
            },
        },
    )
    assert [i.name for i in collection.walk()] == ["foo", "bar", "baz", "one", "two"]
    assert [i.path for i in collection.walk()] == [
        "foo",
        "bar",
        "baz",
        "baz/one",
        "baz/two",
    ]


def test_getitem_folder(httpx_mock, collection):
    httpx_mock.add_response(
        url=(
            "https://pigeon.test/api/v1/collections/xyz/LIVE"
            "/items?name=foo&in_folder=ROOT"
        ),
        json={
            "items": [
                {
                    "id": "f",
                    "name": "foo",
                    "type": "file",
                    "createdOn": datetime.now().isoformat(),
                }
            ],
            "pagination": {
                "total_items": 1,
                "items_per_page": 50,
                "next_page_url": None,
            },
        },
    )
    assert collection["foo"].name == "foo"

    httpx_mock.add_response(
        url=(
            "https://pigeon.test/api/v1/collections/xyz/LIVE"
            "/items?name=baz&type=folder&in_folder=ROOT"
        ),
        json={
            "items": [
                {
                    "id": "z",
                    "name": "baz",
                    "type": "folder",
                    "createdOn": datetime.now().isoformat(),
                }
            ],
            "pagination": {
                "total_items": 1,
                "items_per_page": 50,
                "next_page_url": None,
            },
        },
    )
    httpx_mock.add_response(
        url=(
            "https://pigeon.test/api/v1/collections/xyz/LIVE"
            "/items?name=one&in_path=baz"
        ),
        json={
            "items": [
                {
                    "id": "1",
                    "name": "one",
                    "type": "file",
                    "createdOn": datetime.now().isoformat(),
                }
            ],
            "pagination": {
                "total_items": 1,
                "items_per_page": 50,
                "next_page_url": None,
            },
        },
    )
    assert collection["baz/one"].name == "one"


def test_get_table(httpx_mock, collection):
    httpx_mock.add_response(
        url=(
            "https://pigeon.test/api/v1/collections/xyz/LIVE"
            "/items?name=foo&in_folder=ROOT"
        ),
        json={
            "items": [
                {
                    "id": "f",
                    "name": "foo",
                    "type": "file",
                    "createdOn": datetime.now().isoformat(),
                }
            ],
            "pagination": {
                "total_items": 1,
                "items_per_page": 50,
                "next_page_url": None,
            },
        },
    )
    httpx_mock.add_response(
        url=(
            "https://pigeon.test/api/v1/collections/xyz/LIVE"
            "/tables/f/data?page_size=10000"
        ),
        json={
            "errors": [],
            "pagination": {
                "total_items": 5,
                "items_per_page": 10000,
                "next_page_url": None,
            },
            "data": [
                {"a": 1, "b": 1, "c": 1},
                {"a": 2, "b": 4, "c": 8},
                {"a": 3, "b": 9, "c": 27},
                {"a": 4, "b": 16, "c": 64},
                {"a": 5, "b": 25, "c": 125},
            ],
            "data_model": {
                "type": "object",
                "properties": {
                    "a": {"type": "number"},
                    "b": {"type": "number"},
                    "c": {"type": "number"},
                },
            },
        },
    )
    df = collection.get_table("foo")
    assert len(df) == 5


def test_write_table(httpx_mock, collection):
    httpx_mock.add_response(
        method="POST",
        url="https://pigeon.test/api/v1/collections/xyz/LIVE/items",
        status_code=201,
        json={
            "id": "b",
            "name": "bar",
            "type": "file",
            "createdOn": datetime.now().isoformat(),
        },
    )
    df = pd.DataFrame({"A": [1, 2, 3], "B": [1, 4, 9]})
    collection.write_table("bar", df)

    request = httpx_mock.get_request()
    request.read()
    assert b'{"name": "csv"}' in request.content
    assert b"A,B\n1,1\n2,4\n3,9\n" in request.content



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.prql_module import PrqlModule


T = TypeVar("T", bound="ParsersResolvePrqlModulesResponse200")


@_attrs_define
class ParsersResolvePrqlModulesResponse200:
    """ParsersResolvePrqlModulesResponse200 model

    Attributes:
        modules (List['PrqlModule']):
    """

    modules: List["PrqlModule"]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        modules = []
        for modules_item_data in self.modules:
            modules_item = modules_item_data.to_dict()
            modules.append(modules_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "modules": modules,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`ParsersResolvePrqlModulesResponse200` from a dict"""
        d = src_dict.copy()
        modules = []
        _modules = d.pop("modules")
        for modules_item_data in _modules:
            modules_item = PrqlModule.from_dict(modules_item_data)

            modules.append(modules_item)

        parsers_resolve_prql_modules_response_200 = cls(
            modules=modules,
        )

        return parsers_resolve_prql_modules_response_200



from enum import Enum


class ItemType(str, Enum):
    DATAVIEW = "dataview"
    DICTIONARY = "dictionary"
    FILE = "file"
    FOLDER = "folder"
    LINK = "link"

    def __str__(self) -> str:
        return str(self.value)



from pypigeon.pigeon_core.api.auth import auth_get_session
from pypigeon.pigeon_core.models import AuthGetSessionResponse200

from .base_commands import BaseCommands


class AuthCommands(BaseCommands):
    """Authentication and authorization operations"""

    def whoami(self) -> None:
        """Information about your current session"""
        session = auth_get_session.sync(client=self.core)
        if not isinstance(session, AuthGetSessionResponse200):
            raise Exception("could not read session")

        self._output(session.to_dict())



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.item_storage_checksum import ItemStorageChecksum
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="ItemStorage")


@_attrs_define
class ItemStorage:
    """ItemStorage model

    Example:

    >>> model = ItemStorage.from_dict({'checksum': {},
    ...     'size': None,
    ...     'sizeIsEstimate': False,
    ...     'url': 'pigeon+datastore://default'})

    Attributes:
        checksum (Union[Unset, ItemStorageChecksum]):
        size (Union[Unset, int]):
        size_is_estimate (Union[Unset, bool]):  Default: False.
        url (Union[Unset, str]):
    """

    checksum: Union[Unset, "ItemStorageChecksum"] = UNSET
    size: Union[Unset, int] = UNSET
    size_is_estimate: Union[Unset, bool] = False
    url: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        checksum: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.checksum, Unset):
            checksum = self.checksum.to_dict()
        size = self.size
        size_is_estimate = self.size_is_estimate
        url = self.url

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if checksum is not UNSET:
            field_dict["checksum"] = checksum
        if size is not UNSET:
            field_dict["size"] = size
        if size_is_estimate is not UNSET:
            field_dict["sizeIsEstimate"] = size_is_estimate
        if url is not UNSET:
            field_dict["url"] = url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`ItemStorage` from a dict"""
        d = src_dict.copy()
        _checksum = d.pop("checksum", UNSET)
        checksum: Union[Unset, ItemStorageChecksum]
        if isinstance(_checksum, Unset):
            checksum = UNSET
        else:
            checksum = ItemStorageChecksum.from_dict(_checksum)

        size = d.pop("size", UNSET)

        size_is_estimate = d.pop("sizeIsEstimate", UNSET)

        url = d.pop("url", UNSET)

        item_storage = cls(
            checksum=checksum,
            size=size,
            size_is_estimate=size_is_estimate,
            url=url,
        )

        return item_storage



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.dictionary_search_options_options_additional_property_type import (
    DictionarySearchOptionsOptionsAdditionalPropertyType,
)
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="DictionarySearchOptionsOptionsAdditionalProperty")


@_attrs_define
class DictionarySearchOptionsOptionsAdditionalProperty:
    """DictionarySearchOptionsOptionsAdditionalProperty model

    Attributes:
        type (DictionarySearchOptionsOptionsAdditionalPropertyType):
        default (Union[Unset, Any]):
    """

    type: DictionarySearchOptionsOptionsAdditionalPropertyType
    default: Union[Unset, Any] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        type = self.type.value
        default = self.default

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "type": type,
            }
        )
        if default is not UNSET:
            field_dict["default"] = default

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`DictionarySearchOptionsOptionsAdditionalProperty` from a dict"""
        d = src_dict.copy()
        type = DictionarySearchOptionsOptionsAdditionalPropertyType(d.pop("type"))

        default = d.pop("default", UNSET)

        dictionary_search_options_options_additional_property = cls(
            type=type,
            default=default,
        )

        return dictionary_search_options_options_additional_property



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar("T", bound="PrqlModule")


@_attrs_define
class PrqlModule:
    """PrqlModule model

    Attributes:
        module_contents (str):
        name (str):
    """

    module_contents: str
    name: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        module_contents = self.module_contents
        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "module_contents": module_contents,
                "name": name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`PrqlModule` from a dict"""
        d = src_dict.copy()
        module_contents = d.pop("module_contents")

        name = d.pop("name")

        prql_module = cls(
            module_contents=module_contents,
            name=name,
        )

        return prql_module



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.federation_user_type import FederationUserType
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="FederationUser")


@_attrs_define
class FederationUser:
    """FederationUser model

    Attributes:
        followers (Union[Unset, str]):
        following (Union[Unset, str]):
        inbox (Union[Unset, str]):
        likes (Union[Unset, str]):
        outbox (Union[Unset, str]):
        preferred_username (Union[Unset, str]):
        type (Union[Unset, FederationUserType]):
    """

    followers: Union[Unset, str] = UNSET
    following: Union[Unset, str] = UNSET
    inbox: Union[Unset, str] = UNSET
    likes: Union[Unset, str] = UNSET
    outbox: Union[Unset, str] = UNSET
    preferred_username: Union[Unset, str] = UNSET
    type: Union[Unset, FederationUserType] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        followers = self.followers
        following = self.following
        inbox = self.inbox
        likes = self.likes
        outbox = self.outbox
        preferred_username = self.preferred_username
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if followers is not UNSET:
            field_dict["followers"] = followers
        if following is not UNSET:
            field_dict["following"] = following
        if inbox is not UNSET:
            field_dict["inbox"] = inbox
        if likes is not UNSET:
            field_dict["likes"] = likes
        if outbox is not UNSET:
            field_dict["outbox"] = outbox
        if preferred_username is not UNSET:
            field_dict["preferredUsername"] = preferred_username
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`FederationUser` from a dict"""
        d = src_dict.copy()
        followers = d.pop("followers", UNSET)

        following = d.pop("following", UNSET)

        inbox = d.pop("inbox", UNSET)

        likes = d.pop("likes", UNSET)

        outbox = d.pop("outbox", UNSET)

        preferred_username = d.pop("preferredUsername", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, FederationUserType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = FederationUserType(_type)

        federation_user = cls(
            followers=followers,
            following=following,
            inbox=inbox,
            likes=likes,
            outbox=outbox,
            preferred_username=preferred_username,
            type=type,
        )

        federation_user.additional_properties = d
        return federation_user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar("T", bound="TasksStats")


@_attrs_define
class TasksStats:
    """TasksStats model

    Attributes:
        queued (int):
        results (int):
        schedules (int):
    """

    queued: int
    results: int
    schedules: int

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        queued = self.queued
        results = self.results
        schedules = self.schedules

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "queued": queued,
                "results": results,
                "schedules": schedules,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`TasksStats` from a dict"""
        d = src_dict.copy()
        queued = d.pop("queued")

        results = d.pop("results")

        schedules = d.pop("schedules")

        tasks_stats = cls(
            queued=queued,
            results=results,
            schedules=schedules,
        )

        return tasks_stats



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.collection_access_level import CollectionAccessLevel
from ..models.collection_authorization_update_access_level_action import (
    CollectionAuthorizationUpdateAccessLevelAction,
)
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="CollectionAuthorizationUpdateAccessLevel")


@_attrs_define
class CollectionAuthorizationUpdateAccessLevel:
    """CollectionAuthorizationUpdateAccessLevel model

    Attributes:
        action (CollectionAuthorizationUpdateAccessLevelAction):
        new_level (CollectionAccessLevel):
        controlled_group (Union[Unset, str]):
    """

    action: CollectionAuthorizationUpdateAccessLevelAction
    new_level: CollectionAccessLevel
    controlled_group: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        action = self.action.value
        new_level = self.new_level.value
        controlled_group = self.controlled_group

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "action": action,
                "newLevel": new_level,
            }
        )
        if controlled_group is not UNSET:
            field_dict["controlledGroup"] = controlled_group

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionAuthorizationUpdateAccessLevel` from a dict"""
        d = src_dict.copy()
        action = CollectionAuthorizationUpdateAccessLevelAction(d.pop("action"))

        new_level = CollectionAccessLevel(d.pop("newLevel"))

        controlled_group = d.pop("controlledGroup", UNSET)

        collection_authorization_update_access_level = cls(
            action=action,
            new_level=new_level,
            controlled_group=controlled_group,
        )

        return collection_authorization_update_access_level



"""List items or find items by properties"""
from http import HTTPStatus
from typing import Any
from typing import Dict
from typing import List
from typing import Union

import httpx

from ... import errors
from ...client import AuthenticatedClient
from ...client import Client
from ...models.collections_items_list_items_response_200 import (
    CollectionsItemsListItemsResponse200,
)
from ...models.item_type import ItemType
from ...models.server_error import ServerError
from ...types import Response
from ...types import UNSET
from ...types import Unset


def _get_kwargs(
    collection_id: str,
    version_id: str,
    *,
    name: Union[Unset, str] = UNSET,
    type: Union[Unset, List[ItemType]] = UNSET,
    in_folder: Union[Unset, str] = UNSET,
    in_path: Union[Unset, str] = UNSET,
    parsable: Union[Unset, bool] = UNSET,
    is_failed: Union[Unset, bool] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["name"] = name

    json_type: Union[Unset, List[str]] = UNSET
    if not isinstance(type, Unset):
        json_type = []
        for type_item_data in type:
            type_item = type_item_data.value
            json_type.append(type_item)

    params["type"] = json_type

    params["in_folder"] = in_folder

    params["in_path"] = in_path

    params["parsable"] = parsable

    params["is_failed"] = is_failed

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/collections/{collection_id}/{version_id}/items".format(
            collection_id=collection_id,
            version_id=version_id,
        ),
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Union[CollectionsItemsListItemsResponse200, ServerError]:
    if response.status_code == HTTPStatus.OK:
        response_200 = CollectionsItemsListItemsResponse200.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = ServerError.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = ServerError.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.UNSUPPORTED_MEDIA_TYPE:
        response_415 = ServerError.from_dict(response.json())

        return response_415
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = ServerError.from_dict(response.json())

        return response_500

    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[CollectionsItemsListItemsResponse200, ServerError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    collection_id: str,
    version_id: str,
    *,
    client: AuthenticatedClient,
    name: Union[Unset, str] = UNSET,
    type: Union[Unset, List[ItemType]] = UNSET,
    in_folder: Union[Unset, str] = UNSET,
    in_path: Union[Unset, str] = UNSET,
    parsable: Union[Unset, bool] = UNSET,
    is_failed: Union[Unset, bool] = UNSET,
) -> Response[Union[CollectionsItemsListItemsResponse200, ServerError]]:
    """List items or find items by properties

    Find one or more items as part of the collection (and version ID).

    Query filters are combined with the AND operator. Multiple
    values may be supplied for each filter and will be combined
    with the OR operator.

    The resulting list of items will include the storage, parser,
    and status fields, but the columns field will always be `null`
    since some items may have a large number of columns.

    Args:
        collection_id (str):
        version_id (str):
        name (Union[Unset, str]):
        type (Union[Unset, List[ItemType]]):
        in_folder (Union[Unset, str]):
        in_path (Union[Unset, str]):
        parsable (Union[Unset, bool]):
        is_failed (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[CollectionsItemsListItemsResponse200, ServerError]]
    """

    kwargs = _get_kwargs(
        collection_id=collection_id,
        version_id=version_id,
        name=name,
        type=type,
        in_folder=in_folder,
        in_path=in_path,
        parsable=parsable,
        is_failed=is_failed,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    collection_id: str,
    version_id: str,
    *,
    client: AuthenticatedClient,
    name: Union[Unset, str] = UNSET,
    type: Union[Unset, List[ItemType]] = UNSET,
    in_folder: Union[Unset, str] = UNSET,
    in_path: Union[Unset, str] = UNSET,
    parsable: Union[Unset, bool] = UNSET,
    is_failed: Union[Unset, bool] = UNSET,
) -> Union[CollectionsItemsListItemsResponse200]:
    """List items or find items by properties

    Find one or more items as part of the collection (and version ID).

    Query filters are combined with the AND operator. Multiple
    values may be supplied for each filter and will be combined
    with the OR operator.

    The resulting list of items will include the storage, parser,
    and status fields, but the columns field will always be `null`
    since some items may have a large number of columns.

    Args:
        collection_id (str):
        version_id (str):
        name (Union[Unset, str]):
        type (Union[Unset, List[ItemType]]):
        in_folder (Union[Unset, str]):
        in_path (Union[Unset, str]):
        parsable (Union[Unset, bool]):
        is_failed (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[CollectionsItemsListItemsResponse200]
    """

    response = sync_detailed(
        collection_id=collection_id,
        version_id=version_id,
        client=client,
        name=name,
        type=type,
        in_folder=in_folder,
        in_path=in_path,
        parsable=parsable,
        is_failed=is_failed,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed


async def asyncio_detailed(
    collection_id: str,
    version_id: str,
    *,
    client: AuthenticatedClient,
    name: Union[Unset, str] = UNSET,
    type: Union[Unset, List[ItemType]] = UNSET,
    in_folder: Union[Unset, str] = UNSET,
    in_path: Union[Unset, str] = UNSET,
    parsable: Union[Unset, bool] = UNSET,
    is_failed: Union[Unset, bool] = UNSET,
) -> Response[Union[CollectionsItemsListItemsResponse200, ServerError]]:
    """List items or find items by properties

    Find one or more items as part of the collection (and version ID).

    Query filters are combined with the AND operator. Multiple
    values may be supplied for each filter and will be combined
    with the OR operator.

    The resulting list of items will include the storage, parser,
    and status fields, but the columns field will always be `null`
    since some items may have a large number of columns.

    Args:
        collection_id (str):
        version_id (str):
        name (Union[Unset, str]):
        type (Union[Unset, List[ItemType]]):
        in_folder (Union[Unset, str]):
        in_path (Union[Unset, str]):
        parsable (Union[Unset, bool]):
        is_failed (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[CollectionsItemsListItemsResponse200, ServerError]]
    """

    kwargs = _get_kwargs(
        collection_id=collection_id,
        version_id=version_id,
        name=name,
        type=type,
        in_folder=in_folder,
        in_path=in_path,
        parsable=parsable,
        is_failed=is_failed,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    collection_id: str,
    version_id: str,
    *,
    client: AuthenticatedClient,
    name: Union[Unset, str] = UNSET,
    type: Union[Unset, List[ItemType]] = UNSET,
    in_folder: Union[Unset, str] = UNSET,
    in_path: Union[Unset, str] = UNSET,
    parsable: Union[Unset, bool] = UNSET,
    is_failed: Union[Unset, bool] = UNSET,
) -> Union[CollectionsItemsListItemsResponse200]:
    """List items or find items by properties

    Find one or more items as part of the collection (and version ID).

    Query filters are combined with the AND operator. Multiple
    values may be supplied for each filter and will be combined
    with the OR operator.

    The resulting list of items will include the storage, parser,
    and status fields, but the columns field will always be `null`
    since some items may have a large number of columns.

    Args:
        collection_id (str):
        version_id (str):
        name (Union[Unset, str]):
        type (Union[Unset, List[ItemType]]):
        in_folder (Union[Unset, str]):
        in_path (Union[Unset, str]):
        parsable (Union[Unset, bool]):
        is_failed (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[CollectionsItemsListItemsResponse200]
    """

    response = await asyncio_detailed(
        collection_id=collection_id,
        version_id=version_id,
        client=client,
        name=name,
        type=type,
        in_folder=in_folder,
        in_path=in_path,
        parsable=parsable,
        is_failed=is_failed,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed



from enum import Enum


class AdminOperationsItem(str, Enum):
    ADMIN_CDE = "admin.cde"
    ADMIN_CONFIG = "admin.config"
    ADMIN_GRANTADMIN = "admin.grantadmin"
    ADMIN_METADATA = "admin.metadata"
    ADMIN_TASKS = "admin.tasks"
    ADMIN_USERS = "admin.users"

    def __str__(self) -> str:
        return str(self.value)



"""Create a new data element reference in a data dictionary"""
from http import HTTPStatus
from typing import Any
from typing import Dict
from typing import Union

import httpx

from ... import errors
from ...client import AuthenticatedClient
from ...client import Client
from ...models.data_element import DataElement
from ...models.data_element_reference import DataElementReference
from ...models.server_error import ServerError
from ...types import Response


def _get_kwargs(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    body: DataElementReference,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": "/collections/{collection_id}/{version_id}/dictionaries/{item_id}/refs".format(
            collection_id=collection_id,
            version_id=version_id,
            item_id=item_id,
        ),
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Union[DataElement, ServerError]:
    if response.status_code == HTTPStatus.CREATED:
        response_201 = DataElement.from_dict(response.json())

        return response_201
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = ServerError.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = ServerError.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = ServerError.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = ServerError.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.UNSUPPORTED_MEDIA_TYPE:
        response_415 = ServerError.from_dict(response.json())

        return response_415
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = ServerError.from_dict(response.json())

        return response_500

    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[DataElement, ServerError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
    body: DataElementReference,
) -> Response[Union[DataElement, ServerError]]:
    """Create a new data element reference in a data dictionary

    TODO

    Args:
        collection_id (str):
        version_id (str):
        item_id (str):
        body (DataElementReference): URL-based reference to a Pigeon Data Element

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[DataElement, ServerError]]
    """

    kwargs = _get_kwargs(
        collection_id=collection_id,
        version_id=version_id,
        item_id=item_id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
    body: DataElementReference,
) -> Union[DataElement]:
    """Create a new data element reference in a data dictionary

    TODO

    Args:
        collection_id (str):
        version_id (str):
        item_id (str):
        body (DataElementReference): URL-based reference to a Pigeon Data Element

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[DataElement]
    """

    response = sync_detailed(
        collection_id=collection_id,
        version_id=version_id,
        item_id=item_id,
        client=client,
        body=body,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed


async def asyncio_detailed(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
    body: DataElementReference,
) -> Response[Union[DataElement, ServerError]]:
    """Create a new data element reference in a data dictionary

    TODO

    Args:
        collection_id (str):
        version_id (str):
        item_id (str):
        body (DataElementReference): URL-based reference to a Pigeon Data Element

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[DataElement, ServerError]]
    """

    kwargs = _get_kwargs(
        collection_id=collection_id,
        version_id=version_id,
        item_id=item_id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
    body: DataElementReference,
) -> Union[DataElement]:
    """Create a new data element reference in a data dictionary

    TODO

    Args:
        collection_id (str):
        version_id (str):
        item_id (str):
        body (DataElementReference): URL-based reference to a Pigeon Data Element

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[DataElement]
    """

    response = await asyncio_detailed(
        collection_id=collection_id,
        version_id=version_id,
        item_id=item_id,
        client=client,
        body=body,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed



from datetime import datetime

from pypigeon.client import PigeonClient


def test_get_collection_by_name(httpx_mock, client):
    httpx_mock.add_response(
        url="https://pigeon.test/api/v1/collections?name=test&page_size=50",
        json={
            "collections": [
                {
                    "id": "xyz",
                    "versionId": "LIVE",
                    "accessLevel": "private",
                    "account_id": "aaa",
                    "createdOn": datetime.now().isoformat(),
                    "description": "",
                    "name": "XYZ",
                    "metadata": {},
                }
            ],
            "pagination": {
                "total_items": 1,
                "items_per_page": 50,
                "next_page_url": None,
            },
        },
    )
    p = PigeonClient(client)
    collection = p.get_collection_by_name("test")
    assert collection.id == "xyz"



import json
from io import BytesIO
from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Tuple
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.collections_items_create_item_files_form_style_upload_metadata import (
    CollectionsItemsCreateItemFilesFormStyleUploadMetadata,
)
from ..models.item_column import ItemColumn
from ..models.item_parser import ItemParser
from ..models.item_storage import ItemStorage
from ..models.item_type import ItemType
from ..types import File
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="CollectionsItemsCreateItemFilesFormStyleUpload")


@_attrs_define
class CollectionsItemsCreateItemFilesFormStyleUpload:
    """Creation of a new item

    Attributes:
        content (File):
        folder_id (str):  Example: ROOT.
        type (ItemType):
        columns (Union[List['ItemColumn'], None, Unset]):
        metadata (Union[Unset, CollectionsItemsCreateItemFilesFormStyleUploadMetadata]):
        parser (Union['ItemParser', None, Unset]):  Example: {'name': 'auto'}.
        storage (Union['ItemStorage', None, Unset]):  Example: {'checksum': {}, 'size': None, 'sizeIsEstimate': False,
            'url': 'pigeon+datastore://default'}.
    """

    content: File
    folder_id: str
    type: ItemType
    columns: Union[List["ItemColumn"], None, Unset] = UNSET
    metadata: Union[
        Unset, "CollectionsItemsCreateItemFilesFormStyleUploadMetadata"
    ] = UNSET
    parser: Union["ItemParser", None, Unset] = UNSET
    storage: Union["ItemStorage", None, Unset] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        content = self.content.to_tuple()

        folder_id = self.folder_id
        type = self.type.value
        columns: Union[List[Dict[str, Any]], None, Unset]
        if isinstance(self.columns, Unset):
            columns = UNSET
        elif isinstance(self.columns, list):
            columns = []
            for componentsschemasitem_columns_type_0_item_data in self.columns:
                componentsschemasitem_columns_type_0_item = (
                    componentsschemasitem_columns_type_0_item_data.to_dict()
                )
                columns.append(componentsschemasitem_columns_type_0_item)

        else:
            columns = self.columns
        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()
        parser: Union[Dict[str, Any], None, Unset]
        if isinstance(self.parser, Unset):
            parser = UNSET
        elif isinstance(self.parser, ItemParser):
            parser = self.parser.to_dict()
        else:
            parser = self.parser
        storage: Union[Dict[str, Any], None, Unset]
        if isinstance(self.storage, Unset):
            storage = UNSET
        elif isinstance(self.storage, ItemStorage):
            storage = self.storage.to_dict()
        else:
            storage = self.storage

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "content": content,
                "folderId": folder_id,
                "type": type,
            }
        )
        if columns is not UNSET:
            field_dict["columns"] = columns
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if parser is not UNSET:
            field_dict["parser"] = parser
        if storage is not UNSET:
            field_dict["storage"] = storage

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        content = self.content.to_tuple()

        folder_id = (
            self.folder_id
            if isinstance(self.folder_id, Unset)
            else (None, str(self.folder_id).encode(), "text/plain")
        )
        type = (None, str(self.type.value).encode(), "text/plain")
        columns: Union[None, Tuple[None, bytes, str], Unset]
        if isinstance(self.columns, Unset):
            columns = UNSET
        elif isinstance(self.columns, list):
            _temp_columns = []
            for componentsschemasitem_columns_type_0_item_data in self.columns:
                componentsschemasitem_columns_type_0_item = (
                    componentsschemasitem_columns_type_0_item_data.to_dict()
                )
                _temp_columns.append(componentsschemasitem_columns_type_0_item)
            columns = (None, json.dumps(_temp_columns).encode(), "application/json")

        else:
            columns = self.columns
        metadata: Union[Unset, Tuple[None, bytes, str]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = (
                None,
                json.dumps(self.metadata.to_dict()).encode(),
                "application/json",
            )
        parser: Union[None, Tuple[None, bytes, str], Unset]
        if isinstance(self.parser, Unset):
            parser = UNSET
        elif isinstance(self.parser, ItemParser):
            parser = (
                None,
                json.dumps(self.parser.to_dict()).encode(),
                "application/json",
            )
        else:
            parser = self.parser
        storage: Union[None, Tuple[None, bytes, str], Unset]
        if isinstance(self.storage, Unset):
            storage = UNSET
        elif isinstance(self.storage, ItemStorage):
            storage = (
                None,
                json.dumps(self.storage.to_dict()).encode(),
                "application/json",
            )
        else:
            storage = self.storage

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "content": content,
                "folderId": folder_id,
                "type": type,
            }
        )
        if columns is not UNSET:
            field_dict["columns"] = columns
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if parser is not UNSET:
            field_dict["parser"] = parser
        if storage is not UNSET:
            field_dict["storage"] = storage

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionsItemsCreateItemFilesFormStyleUpload` from a dict"""
        d = src_dict.copy()
        content = File(payload=BytesIO(d.pop("content")))

        folder_id = d.pop("folderId")

        type = ItemType(d.pop("type"))

        def _parse_columns(data: object) -> Union[List["ItemColumn"], None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                componentsschemasitem_columns_type_0 = []
                _componentsschemasitem_columns_type_0 = data
                for (
                    componentsschemasitem_columns_type_0_item_data
                ) in _componentsschemasitem_columns_type_0:
                    componentsschemasitem_columns_type_0_item = ItemColumn.from_dict(
                        componentsschemasitem_columns_type_0_item_data
                    )

                    componentsschemasitem_columns_type_0.append(
                        componentsschemasitem_columns_type_0_item
                    )

                return componentsschemasitem_columns_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List["ItemColumn"], None, Unset], data)

        columns = _parse_columns(d.pop("columns", UNSET))

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, CollectionsItemsCreateItemFilesFormStyleUploadMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = CollectionsItemsCreateItemFilesFormStyleUploadMetadata.from_dict(
                _metadata
            )

        def _parse_parser(data: object) -> Union["ItemParser", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemasitem_parser_type_0 = ItemParser.from_dict(data)

                return componentsschemasitem_parser_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ItemParser", None, Unset], data)

        parser = _parse_parser(d.pop("parser", UNSET))

        def _parse_storage(data: object) -> Union["ItemStorage", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemasitem_storage_type_0 = ItemStorage.from_dict(data)

                return componentsschemasitem_storage_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ItemStorage", None, Unset], data)

        storage = _parse_storage(d.pop("storage", UNSET))

        collections_items_create_item_files_form_style_upload = cls(
            content=content,
            folder_id=folder_id,
            type=type,
            columns=columns,
            metadata=metadata,
            parser=parser,
            storage=storage,
        )

        return collections_items_create_item_files_form_style_upload



from enum import Enum


class CollectionsTablesGetFormattedTableDataExportFormat(str, Enum):
    CSV = "csv"
    EXCEL = "excel"
    PARQUET = "parquet"
    TSV = "tsv"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.collection_authorization_remove_identity import (
    CollectionAuthorizationRemoveIdentity,
)
from ..models.collection_authorization_set_identity import (
    CollectionAuthorizationSetIdentity,
)
from ..models.collection_authorization_update_access_level import (
    CollectionAuthorizationUpdateAccessLevel,
)


T = TypeVar("T", bound="CollectionAuthorizationChange")


@_attrs_define
class CollectionAuthorizationChange:
    """CollectionAuthorizationChange model

    Attributes:
        req (Union['CollectionAuthorizationRemoveIdentity', 'CollectionAuthorizationSetIdentity',
            'CollectionAuthorizationUpdateAccessLevel']):
    """

    req: Union[
        "CollectionAuthorizationRemoveIdentity",
        "CollectionAuthorizationSetIdentity",
        "CollectionAuthorizationUpdateAccessLevel",
    ]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        req: Dict[str, Any]
        if isinstance(self.req, CollectionAuthorizationSetIdentity):
            req = self.req.to_dict()
        elif isinstance(self.req, CollectionAuthorizationRemoveIdentity):
            req = self.req.to_dict()
        else:
            req = self.req.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "req": req,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionAuthorizationChange` from a dict"""
        d = src_dict.copy()

        def _parse_req(
            data: object,
        ) -> Union[
            "CollectionAuthorizationRemoveIdentity",
            "CollectionAuthorizationSetIdentity",
            "CollectionAuthorizationUpdateAccessLevel",
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                req_type_0 = CollectionAuthorizationSetIdentity.from_dict(data)

                return req_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                req_type_1 = CollectionAuthorizationRemoveIdentity.from_dict(data)

                return req_type_1
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            req_type_2 = CollectionAuthorizationUpdateAccessLevel.from_dict(data)

            return req_type_2

        req = _parse_req(d.pop("req"))

        collection_authorization_change = cls(
            req=req,
        )

        return collection_authorization_change



from enum import Enum


class DataDictionarySourceItem(str, Enum):
    CDE = "cde"
    EXTERNAL = "external"
    REFERENCEDINTHISCOLLECTION = "referencedInThisCollection"
    SYSTEMSTANDARD = "systemStandard"
    THISCOLLECTION = "thisCollection"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.search_search_collections_body_facets import (
    SearchSearchCollectionsBodyFacets,
)
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="SearchSearchCollectionsBody")


@_attrs_define
class SearchSearchCollectionsBody:
    """SearchSearchCollectionsBody model

    Attributes:
        query (str):
        facets (Union[Unset, SearchSearchCollectionsBodyFacets]):
    """

    query: str
    facets: Union[Unset, "SearchSearchCollectionsBodyFacets"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        query = self.query
        facets: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.facets, Unset):
            facets = self.facets.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "query": query,
            }
        )
        if facets is not UNSET:
            field_dict["facets"] = facets

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SearchSearchCollectionsBody` from a dict"""
        d = src_dict.copy()
        query = d.pop("query")

        _facets = d.pop("facets", UNSET)
        facets: Union[Unset, SearchSearchCollectionsBodyFacets]
        if isinstance(_facets, Unset):
            facets = UNSET
        else:
            facets = SearchSearchCollectionsBodyFacets.from_dict(_facets)

        search_search_collections_body = cls(
            query=query,
            facets=facets,
        )

        return search_search_collections_body



import datetime
from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define
from dateutil.parser import isoparse


T = TypeVar("T", bound="Group")


@_attrs_define
class Group:
    """Group model

    Attributes:
        created_on (datetime.datetime):
        id (str):
        name (str):
    """

    created_on: datetime.datetime
    id: str
    name: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        created_on = self.created_on.isoformat()
        id = self.id
        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "createdOn": created_on,
                "id": id,
                "name": name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`Group` from a dict"""
        d = src_dict.copy()
        created_on = isoparse(d.pop("createdOn"))

        id = d.pop("id")

        name = d.pop("name")

        group = cls(
            created_on=created_on,
            id=id,
            name=name,
        )

        return group
